#include "globals.h"
#include "Texture.h"

extern bool bilerp_flag;

Texture :: Texture(char *filename)
{
//http://stackoverflow.com/questions/2693631/read-ppm-file-and-store-it-in-an-array-coded-with-c
   char buf[30];
   int rgbSize;
   FILE *fp;

   fp = fopen(filename, "rb");
   if (!fp)
   {
      fprintf(stderr, "ERROR! Could not read texture %s\n", filename);
      fflush(stderr);
      exit(-1);
   }
   //fprintf(stderr, "Opened texture %s\n", filename);

   /* read image format string "P6" */
   fscanf(fp, "%s", buf);

   /* read IrfanView comment */
   fscanf(fp, "%s%s%s%s", buf, buf, buf, buf);

   /* read image dimensions */
   fscanf(fp, "%d%d", &width, &height);
   fprintf(stderr, "Texture %s has dimensions width = %d, height = %d\n\n",
                    filename, width, height);

   /* read image rgb dimensions (which we don't use) */
   fscanf(fp, "%d", &rgbSize);
   //fprintf(stderr, "Texture %s has rgb dimensions rgbSize = %d\n",                    filename, rgbSize);

   /* make room for the data array */
   data = (u08 *)malloc(sizeof(u08) * width * height * 3);
   if (!data)
   {
      fprintf(stderr, "ERROR! Unable to allocate memory for texture %s\n", filename);
      fflush(stderr);
      exit(-1);
   }
   //fprintf(stderr, "Allocated %d bytes for texture %s\n", 3*width*height, filename);

   /* Eat line feeds */
   while (fgetc(fp) != '\n') ;

   //read pixel data from file
   if (fread(data, 3 * width, height, fp) != (size_t)height)
   {
      fprintf(stderr, "ERROR! Could not load texture %s\n", filename);
      fflush(stderr);
      exit(-1);
   }
   //fprintf(stderr, "Loaded texture %s\n", filename);

   fclose(fp);



   /* if opengl i should bind it and all that */
   glGenTextures(1, &tex);
   glBindTexture(GL_TEXTURE_2D, tex);
   glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE );
   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

   if (bilerp_flag)
   {
      glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
   }
   else
   {
      glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
   }

   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
   glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);

   glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, width, height, 0, GL_RGB, GL_UNSIGNED_BYTE, data);

   return;
};


/* switches between nearest neighbor and bilinear interpolation */
void Texture :: switchTextureFiltering(bool flag)
{
   bilerp_flag = flag;

   glBindTexture(GL_TEXTURE_2D, tex);
   if (bilerp_flag)
   {
      glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
   }
   else
   {
      glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
   }
   return;
}